/*****************************************************************
 * Dummyload projekt 2023 OZ7SKB Skanderborg EDR
 * 
 * Rev. 1.0 22012024  OZ4KAJ
 * 
 * Inspireret af W8TEE Jack Purdum og W6DQ Dennis Kidder
 * Arduino Projects for Amateur Radio 2015
 * 
 * !!!!!!!!!!!!!!!!! ADVARSEL !!!!!!!!!!!!!!!!!
 * Husk at koden skal kompileres og uploades til Arduino Uno, 
 * hvor kun Arduino Uno er tilkoblet. 
 * Alsaa uden display og board med elektronik tilkoblet.
 * RISIKO: Arduino Uno kan blive oedelagt med kortslutning af AREF-porten
 *****************************************************************/
 
// Inkluder hjaelpelibrary
#include <LiquidCrystal.h>

// Konstanter og variabler defineres
/*
// Function prototypes --->>>>> kan undværes
float CalculateTheOutput(int analogIn);
void ShowTheOutput(int analogIn, float val);
void BuildTheOutput(float val);
*/

//#define DEBUG
#define DEFAULTPRECISION    0	  // default decimal places
#define MAXANALOGVALUE   1000   // Set valid input to 0 - 1000
#define VALUEWIDTH          8   // Definere resultatfelt med '-', '.'
#define DIODEVOLTAGEDROP  1.2   // Two 1N4814...måske en anden værdi ved andre dioder
#define DISPLAYDELAY     2000   // Pauser display i 2 sekunder. Forhindre flimmer.

#define UPDATEDELAY       100   // Pause til aflæsning af A1
#define ERRORDELAY        400   // Pause til visning af fejl
#define STARTUPDELAY     3000   // Pause ved opstart til visning af version mm
                                // Tekst er fri, max 2 x 16

#define BARGRAPHROW         1   // Tegner bar-grafen i anden linie
#define BARGRAPHCOL         0   // Start graphen i kolonne 1

const int LCDNUMCOLS =     16;  // Antal kolonner på display
const int LCDNUMROWS =      2;  // Antal linier på display
const int DLSENSORPIN =    A1;  // Definere analog port A1 som input

char buffer[LCDNUMCOLS + 1];    // Buffer til output
float vScaleFactor = 2.0;       // Skalerinsafaktor for Volt. Passer til 100W
float wScaleFactor = 2.0;       // Skalerinsafaktor for watt
const float MESASUREDDUMMYLOADRESISTANCE = 50.4;  // Modstanden i dummyload

// Definere LCD pins
const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);
 
/*********** Initialisering af Uno ***********/
void setup()
{
  Serial.begin(9600);             // Initiere seriel kommunikation

  analogReference(EXTERNAL);      // Brug den eksterne spaendingsreference -- HUSK AT AKTIVERE
  lcd.begin(16, 2);               //  Initiere display til 16 x 2

  #ifdef DEBUG                    
    Serial.println("---------------------------------------------------------------------------");
    Serial.println("OZ7SKB Dummyload - Version 1.0 2024");
  #endif

  // Startup visning på display. Kan frit ændres. Max 16 x 2
  lcd.print("OZ7SKB Dummyload");    // Viser teksten
  lcd.setCursor(0, 1);              // Linje 2
  lcd.print("Version 1.0 2024");    // Viser teksten
  delay(STARTUPDELAY);              //  Hold en pause
  lcd.clear();                      // Display renses  
} // setup() slut

/************ Arbejdsprogrammet gentages uendeligt *********/
void loop()
{
  // Lokale variable
  int analogInVolts;
  float val;

  analogInVolts = analogRead(DLSENSORPIN);      // Aflæs input
  #ifdef DEBUG  
    Serial.println("---------------------------------------------------------------------------");
    Serial.println("analogInVolt = ");
    Serial.println(analogInVolts);

    //analogInVolts = 1000;
  #endif

  if (analogInVolts) {
    BuildTheOutput(analogInVolts);              
    ShowTheOutput(analogInVolts, val); 
    delay(DISPLAYDELAY);
  }
} //loop() stop

/************ BeregnOutput ***********0***0***
 *  Funktionen beregner den maalte effekt
 *  og formattere resultatet til LCD
 *********************************************/
void BuildTheOutput(int voltsIn)
{
  int i;
  float scaledVolts;
  float RMS;
  float watts;

  scaledVolts = (float (voltsIn)) + DIODEVOLTAGEDROP;    // Adjust for diode vvoltage drop

  #ifdef DEBUG  
   Serial.print("scaledVolts1 = ");
   Serial.println(scaledVolts);
  #endif

  scaledVolts = scaledVolts / vScaleFactor;               // Scale it to meter

  #ifdef DEBUG  
   Serial.print("scaledVolts2 = ");
   Serial.println(scaledVolts);
  #endif

  RMS = scaledVolts / 1.414;                             // Get RMS of voltage

  #ifdef DEBUG  
   Serial.print("RMS = ");
   Serial.println(RMS);
  #endif

  watts = (RMS * RMS) / MESASUREDDUMMYLOADRESISTANCE;    // watts into DL
  watts = watts * wScaleFactor;

  #ifdef DEBUG                    
    Serial.print("watts = ");
    Serial.println(watts);
  #endif

  dtostrf(watts, VALUEWIDTH, DEFAULTPRECISION, buffer);  // Convert and format value
  strcat(buffer, " watts");
}


/************ VisOutput *********************0*
 *  Funktionen viser resultatet paa LCD
 **********************************************/
 void ShowTheOutput(int analogIn, float watts)
{
  lcd.setCursor(0,0);                  // Set up the display...
  lcd.print(buffer);

  if (analogIn <= MAXANALOGVALUE) {    // If analog value within scale...
    delay(UPDATEDELAY);    
  } else {                             // ...otherwise complain about it.
    lcd.setCursor(0, 1);
    lcd.print("   Overrange!   ");  
    lcd.noDisplay();                   // flash the display on and off 
    delay(ERRORDELAY);
    lcd.display();
    delay(ERRORDELAY);       
  } 
}
 
